<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.mergado.cz
 * @since      1.0.0
 *
 * @package    Mergado_Marketing_Pack
 * @subpackage Mergado_Marketing_Pack/admin
 */
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Mergado_Marketing_Pack
 * @subpackage Mergado_Marketing_Pack/admin
 * @author     Mergado technologies, s. r. o. <info@mergado.cz>
 */
if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    deactivate_plugins(__FILE__);
    die(WOOCOMMERCE_DEPENCENCY_MESSAGE);
} else {
    if (!defined('WC_ABSPATH')) {
        define('WC_ABSPATH', dirname(__FILE__) . '/../../woocommerce/');
    }
    include_once( WC_ABSPATH . '/includes/export/class-wc-product-csv-exporter.php' );
    if (!class_exists('WC_CSV_Batch_Exporter', false)) {
        include_once( WC_ABSPATH . '/includes/export/abstract-wc-csv-batch-exporter.php' );
    }
    if (!class_exists('WC_CSV_Exporter', false)) {
        include_once( WC_ABSPATH . '/includes/export/abstract-wc-csv-exporter.php' );
    }
}

class Megrado_export extends WC_Product_CSV_Exporter {

    public $limit = 9999999;

    public function generate_data() {

        $this->set_limit($this->limit);
        $this->prepare_data_to_export();
        return $this->row_data;
    }

}

class Mergado_Marketing_Pack_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        self::init_hooks();
    }

    public static function init_hooks() {
        add_action('admin_menu', array('Mergado_Marketing_Pack_Admin', 'admin_menu'));
    }

    public static function admin_menu() {
        self::load_menu();
    }

    public static function load_menu() {
        // Add main menu pages
        add_menu_page(translate('Mergado Marketing Pack', 'mergado'), translate('Mergado', 'mergado'), 'manage_options', 'mergado-config', array('Mergado_Marketing_Pack_Admin', 'display'), 'dashicons-chart-line', 58);
        add_submenu_page('woocommerce', translate('Mergado Marketing Pack', 'mergado'), translate('Mergado Marketing Pack', 'mergado'), 'manage_options', 'mergado-config', array('Mergado_Marketing_Pack_Admin', 'display'));

        // Add submenu pages
        // add_submenu_page('mergado-config', translate('Export configuration', 'mergado'), translate('Export configuration', 'mergado'), 'manage_options', 'mergado-config-export', array( 'Mergado_Marketing_Pack_Admin', 'display' ));
        add_submenu_page('mergado-config', translate('Cron tasks', 'mergado'), translate('Cron tasks', 'mergado'), 'manage_options', 'mergado-cron', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', translate('XML feeds', 'mergado'), translate('XML feeds', 'mergado'), 'manage_options', 'mergado-feed', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', translate('Ad Systems', 'mergado'), translate('Ad Systems', 'mergado'), 'manage_options', 'mergado-adsys', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', translate('Support', 'mergado'), translate('Support', 'mergado'), 'manage_options', 'mergado-support', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', translate('Licence', 'mergado'), translate('Licence', 'mergado'), 'manage_options', 'mergado-licence', array('Mergado_Marketing_Pack_Admin', 'display'));
    }

    public static function generateToken() {
        $token = sha1(get_site_url() . 'MMP2017WP' . uniqid());
        update_option('mmp_token', $token);
        return $token;
    }

    public static function getToken() {
        $token = get_option('mmp_token');
        if ($token === NULL || $token === '') {
            $token = self::generateToken();
        }

        return $token;
    }

    public static function deleteToken() {
        delete_option('mmp_token');
    }

    public static function display() {
        global $wp;
        $token = self::getToken();

        if (isset($_GET['page'])) {
            $page = $_GET['page'];
            if (isset($_GET['action'])) {
                if (isset($_GET['token']) && $_GET['token'] = $token && $token != '' && $token != NULL) {
                    $action = $_GET['action'];
                    switch ($action) {
                        case 'download':
                            self::download($page);
                            break;
                        case 'adminCron':
                            self::cron($page, true);
                            break;
                        case 'buildXml':
                            self::buildXml();
                            break;

                        default:
                            exit;
                            break;
                    }
                } else {
                    wp_redirect('admin.php?page=' . $page);
                }
            } else {
                switch ($page) {
                    case 'mergado-config':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-info.php');
                        break;
                    case 'mergado-cron':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-cron.php');
                        break;
                    case 'mergado-feed':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-feed.php');
                        break;
                    case 'mergado-licence':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-licence.php');
                        break;
                    case 'mergado-support':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-support.php');
                        break;
                    case 'mergado-adsys':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-adsys.php');
                        break;
                    default:
                        exit;
                        break;
                }
            }
        } else {
            die;
        }
    }

    public static function cron($page, $redirect = false) {
        $logger = wc_get_logger();
        $token = self::getToken();
        $file = __DIR__ . '/../data/products_' . $token . '.xml';
        if (!is_dir(__DIR__ . '/../data')) {
            mkdir(__DIR__ . '/../data');
            $logger->info('Mergado log: created data folder for XML');
        }

        $logger->info('Mergado log: product feed generator started');
        $xml = self::createXML();
        $logger->info('Mergado log: product feed generator ended');
        file_put_contents($file, $xml->saveXML());
        $logger->info('Mergado log: product feed generator saved XML file');


        if ($redirect) {
            wp_redirect('admin.php?page=' . $page . '&flash=cronGenerated');
            exit;
        }
    }

    public static function download($page) {
        $logger = wc_get_logger();
        $token = self::getToken();
        $file = __DIR__ . '/../data/products_' . $token . '.xml';
        if (file_exists($file)) {
            $logger->info('Mergado log: XML product file was downloaded');
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($file) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));
            ob_clean();
            flush();
            echo readfile($file);
            exit;
        } else {
            wp_redirect('admin.php?page=' . $page);
        }
    }

    public static function buildXml() {
        $logger = wc_get_logger();
        $token = self::getToken();
        $file = __DIR__ . '/../data/products_' . $token . '.xml';
        $logger->info('Mergado log: product feed generator started');
        $xml = self::createXML();
        $logger->info('Mergado log: product feed generator ended');
        file_put_contents($file, $xml->saveXML());
        $logger->info('Mergado log: XML product file was downloaded');
        header('Content-Description: File Transfer');
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="' . basename($file) . '"');
        header('Expires: 0');
        header('Cache-Control: must-revalidate');
        header('Pragma: public');
        header('Content-Length: ' . filesize($file));
        ob_clean();
        flush();
        echo readfile($file);
        exit;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Mergado_Marketing_Pack_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Mergado_Marketing_Pack_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/mergado-marketing-pack-admin.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Mergado_Marketing_Pack_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Mergado_Marketing_Pack_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/mergado-marketing-pack-admin.js', array('jquery'), $this->version, false);
    }

    protected static function findParams($values) {
        $param = [];
        foreach ($values as $attrName => $attrValue) {
            if (substr($attrName, 0, 11) == 'attributes:') {
                $int = filter_var($attrName, FILTER_SANITIZE_NUMBER_INT);
                $rest = substr($attrName, 11, 0 - strlen($int));
                $params[$int][$rest] = $attrValue;
            }
        }
        return $params;
    }

    protected static function findImages($values) {
        $images = [];
        $exploded = explode(", ", $values);
        $images['main'] = isset($exploded[0]) ? $exploded[0] : '';
        unset($exploded[0]);
        $images['alt'] = array_values($exploded);
        return $images;
    }

    protected static function findCategory($categoryIds) {
        $cateroryGroups = explode(',', $categoryIds);
        foreach ($cateroryGroups as $group) {
            $categoryTrees[] = explode(" > ", $group);
        }
        $counts = array_map('count', $categoryTrees);
        $key = array_flip($counts)[max($counts)];

        return implode(' / ', $categoryTrees[$key]);
    }

    protected static function createXML() {
        global $wpdb;
        $exporter = new Megrado_export();

        $xml = new DOMDocument('1.0', 'UTF-8');
        $channel = $xml->createElement('channel');
        $channel->appendChild($xml->createElement('generator', 'mergado.woocommerce.marketingpack.' . str_replace('.', '_', PLUGIN_VERSION)));


        $params = [];
        $currency = $wpdb->get_col($wpdb->prepare("SELECT option_value FROM $wpdb->options WHERE option_name = %s", 'woocommerce_currency'));
        $defaults['currency'] = array_pop($currency);
        $tax = $wpdb->get_col($wpdb->prepare("SELECT option_value FROM $wpdb->options WHERE option_name = %s", 'woocommerce_calc_taxes'));
        $defaults['tax_calc'] = array_pop($tax) == 'yes' ? true : false;
        $taxInc = $wpdb->get_col($wpdb->prepare("SELECT option_value FROM $wpdb->options WHERE option_name = %s", 'woocommerce_prices_include_tax'));
        $defaults['tax_inc'] = array_pop($taxInc) == 'yes' ? true : false;
        $rates = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}woocommerce_tax_rates");
        $ratesArr = [];
        foreach ($rates as $rate) {
            $ratesArr[$rate->tax_rate_class] = $rate->tax_rate;
        }

        $products = [];
        foreach ($exporter->generate_data() as $k => $v) {
            if ($v['type'] != 'grouped') { // preskoceni slozenych produktu
                $products[$v['id']] = $v;
            }
        }

        foreach ($products as $k => $v) {
            if ($v['type'] != 'variable') {
                $params = self::findParams($v);
                $posts = $wpdb->get_results($wpdb->prepare("SELECT * FROM $wpdb->posts WHERE ID=%s", $v['id']));
                if (isset($posts[0])) {
                    $post = $posts[0];
                } else {
                    continue;
                }

                $item = $xml->createElement('ITEM');

                $item->appendChild($xml->createElement('ITEM_ID', $v['id']));
                $stock = $v['stock_status'] ? 'in stock' : 'out of stock';
                $item->appendChild($xml->createElement('AVAILABILITY', $stock));
                // adresa kde je product
                $url = get_the_permalink($v['id']);
                $item->appendChild($xml->createElement('URL', htmlentities($url)));

                $nameExact = $xml->createElement('NAME_EXACT');
                $nameExact->appendChild($xml->createCDATASection($v['name']));
                $item->appendChild($nameExact);

                $item->appendChild($xml->createElement('CURRENCY', $defaults['currency']));
                $wpPrice = $v['sale_price'] != '' ? $v['sale_price'] : $v['regular_price'];
                if ($defaults['tax_calc']) { // pocitaji se dane
                    $taxRate = $ratesArr[$v['tax_class']];
                    $item->appendChild($xml->createElement('VAT', $taxRate));
                    if ($defaults['tax_inc']) {
                        $koef = $taxRate / (100 + $taxRate);
                        $item->appendChild($xml->createElement('PRICE_VAT', $wpPrice));
                        $item->appendChild($xml->createElement('PRICE', round($wpPrice - ($wpPrice * $koef), 2)));
                    } else {
                        $item->appendChild($xml->createElement('PRICE_VAT', round($wpPrice * (1 + ($taxRate / 100)), 2)));
                        $item->appendChild($xml->createElement('PRICE', $wpPrice));
                    }
                } else { // vubec se nepocitaji dane
                    $item->appendChild($xml->createElement('PRICE_VAT', $wpPrice));
                }
                $images = self::findImages($v['images']);
                $item->appendChild($xml->createElement('IMAGE', $images['main']));

                foreach ($images['alt'] as $img) {
                    $item->appendChild($xml->createElement('IMAGE_ALTERNATIVE', $img));
                }

                if ($params !== null) {
                    foreach ($params as $paramKey => $paramValue) {
                        if ($paramValue['visible']) {
                            $xmlParam = $xml->createElement('PARAM');

                            $paramName = $xml->createElement('NAME');
                            $paramName->appendChild($xml->createCDATASection($paramValue['name']));
                            $xmlParam->appendChild($paramName);

                            $paramNameValue = $xml->createElement('VALUE');
                            $paramNameValue->appendChild($xml->createCDATASection($paramValue['value']));
                            $xmlParam->appendChild($paramNameValue);
                            
                            $item->appendChild($xmlParam);
                        }
                    }
                }

                $item->appendChild($xml->createElement('SHIPPING_SIZE', sprintf('%s x %s x %s cm', $v["length"], $v["width"], $v["height"])));
                $item->appendChild($xml->createElement('SHIPPING_WEIGHT', sprintf('%s kg', $v["weight"])));

                switch ($v['type']) {
                    case 'simple':
                        $productNo = $xml->createElement('PRODUCTNO');
                        $productNo->appendChild($xml->createCDATASection($v['sku']));
                        $item->appendChild($productNo);

                        $description = $xml->createElement('DESCRIPTION');
                        $description->appendChild($xml->createCDATASection($v['description']));
                        $item->appendChild($description);

                        $categories = self::findCategory($v['category_ids']);
                        $category = $xml->createElement('CATEGORY');
                        $category->appendChild($xml->createCDATASection($categories));
                        $item->appendChild($category);

                        $shortDescription = $xml->createElement('DESCRIPTION_SHORT');
                        $shortDescription->appendChild($xml->createCDATASection($v['short_description']));
                        $item->appendChild($shortDescription);

                        break;
                    case 'variation':
                        $parentId = wc_get_product_id_by_sku($v['parent_id']);
                        $item->appendChild($xml->createElement('ITEMGROUP_ID', $parentId));

                        $productNo = $xml->createElement('PRODUCTNO');
                        $productNo->appendChild($xml->createCDATASection(($v['sku'] != '') ? $v['sku'] : (isset($products[$parentId]['sku']) ? $products[$parentId]['sku'] : '')));
                        $item->appendChild($productNo);

                        $categories = self::findCategory($products[$parentId]['category_ids']);
                        $category = $xml->createElement('CATEGORY');
                        $category->appendChild($xml->createCDATASection($categories));
                        $item->appendChild($category);

                        $shortDescription = $xml->createElement('DESCRIPTION_SHORT');
                        $shortDescription->appendChild($xml->createCDATASection($products[$parentId]['short_description']));
                        $item->appendChild($shortDescription);

                        $description = $xml->createElement('DESCRIPTION');
                        $description->appendChild($xml->createCDATASection(sprintf('%s %s', isset($products[$parentId]['description']) ? $products[$parentId]['description'] : '', $v['description'])));

                        $item->appendChild($description);
                        break;

                    default:
                        break;
                }
            }
            $channel->appendChild($item);
        }

        $xml->appendChild($channel);
        return $xml;
    }

}
