<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.mergado.cz
 * @since             1.0.0
 * @package           Mergado_Marketing_Pack
 *
 * @wordpress-plugin
 * Plugin Name:       Mergado marketing pack
 * Plugin URI:        https://www.mergado.cz
 * Description:       Earn more on price comparator sites. <strong>REQUIRES: Woocommerce</strong>
 * Version:           1.0.7
 * Author:            Mergado technologies, s. r. o.
 * Author URI:        https://www.mergado.cz
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       mergado
 * Domain Path:       /languages
 */
// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

define('PLUGIN_VERSION', '1.0.7');
define('WOOCOMMERCE_DEPENCENCY_MESSAGE', __('This plugin requires <a href="/wp-admin/plugin-install.php?tab=plugin-information&plugin=woocommerce" target="_top">WooCommerce</a> plugin to be active!', 'mergado'));

/**
 * Load textdomain
 */
function plugin_name_load_plugin_textdomain() {
    $domain = 'mergado';
    $locale = apply_filters('plugin_locale', get_locale(), $domain);
    load_textdomain($domain, trailingslashit(WP_LANG_DIR) . $domain . '/' . $domain . '-' . $locale . '.mo');
    load_plugin_textdomain($domain, FALSE, basename(dirname(__FILE__)) . '/languages/');
}
add_action('init', 'plugin_name_load_plugin_textdomain');

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-mergado-marketing-pack-activator.php
 */
function activate_mergado_marketing_pack() {
    require_once plugin_dir_path(__FILE__) . 'includes/class-mergado-marketing-pack-activator.php';
    Mergado_Marketing_Pack_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-mergado-marketing-pack-deactivator.php
 */
function deactivate_mergado_marketing_pack() {
    require_once plugin_dir_path(__FILE__) . 'includes/class-mergado-marketing-pack-deactivator.php';
    Mergado_Marketing_Pack_Deactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_mergado_marketing_pack');
register_deactivation_hook(__FILE__, 'deactivate_mergado_marketing_pack');

add_action('wp_loaded', function() {
    $action = isset($_GET['action']) ? $_GET['action'] : '';
    $token = isset($_GET['token']) ? $_GET['token'] : '';

    if ($action !== '' && $token !== '' && add_query_arg(array('action' => NULL, 'token' => NULL)) === '/mergado/') {
        if ($token != Mergado_Marketing_Pack_Admin::getToken()) {
            echo '<span style="display: inline-block; width: 14px; height: 14px; border-radius: 100%; background: red; margin-right: 8px;"></span>';
            _e('ERROR: Invalid token', 'mergado');
            exit;
        }

        switch ($action) {
            case 'cron':
                Mergado_Marketing_Pack_Admin::cron('');
                echo '<span style="display: inline-block; width: 14px; height: 14px; border-radius: 100%; background: green; margin-right: 8px;"></span>';
                _e('SUCCESS: Product feed generated', 'mergado');
                exit;
        }
    }
});

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/class-mergado-marketing-pack.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_mergado_marketing_pack() {
    $plugin = new Mergado_Marketing_Pack();
    $plugin->run();
}

run_mergado_marketing_pack();
